--=====================================================================================
-- LEASE EASY
--
-- @Author:		akuenzi
-- @Date:		2018-08-02
-- @Purpose:	To adjust the lease pricing used in-game, and display time remaining.
--=====================================================================================


--Enter desired lease rates to use in-game.
local baseCost			= 	0.02	--default = 0.02, or 2% of Vehicle cost
local perOperatingHour	=	0.05	--default = 0.05, or 5% of Vehicle cost
local perDay			=	0.01	--default = 0.01, or 1% of Vehicle cost

-- Enter discount rate for what the store pays for fill items contained in lease returns.
local discount			=	0.50
------------------------------------------------------------------------------------------

LeaseEasy = {}

function LeaseEasy:loadMap(name)
	self.firstStart = true
	
end;

function LeaseEasy:deleteMap()
end;

function LeaseEasy:mouseEvent(posX, posY, modifier, isDown)
end;

function LeaseEasy:keyEvent(unicode, sym, modifier, isDown)
end;

function LeaseEasy:update(dt)
	if self.firstStart then
		if baseCost ~= nil and type(baseCost) == "number" then
			if baseCost >= 0 then
				EconomyManager.DEFAULT_LEASING_DEPOSIT_FACTOR = baseCost
			end;
		end;
		if perOperatingHour ~= nil and type(perOperatingHour) == "number" then
			if perOperatingHour >= 0 then
				EconomyManager.DEFAULT_RUNNING_LEASING_FACTOR = perOperatingHour
				for vehicle, _ in pairs(StoreItemsUtil.storeItemsByXMLFilename) do
					StoreItemsUtil.storeItemsByXMLFilename[vehicle].runningLeasingFactor = perOperatingHour
				end;
			end;
		end;
		if perDay ~= nil and type(perDay) == "number" then
			if perDay >= 0 then
				EconomyManager.PER_DAY_LEASING_FACTOR = perDay
			end;
		end;
		
		self.firstStart = false
	end;
end;

function LeaseEasy:draw()
end;

addModEventListener(LeaseEasy);


oldVehicleDraw = Vehicle.draw
Vehicle.draw = function(self)
	
	-- Run existing Vehicle.draw function.
	if oldVehicleDraw ~= nil then
		oldVehicleDraw(self)
	end;
	
	-- If leased vehicle, show time remaining in operating hour.
	if self.isClient then
		if self.propertyState == Vehicle.PROPERTY_STATE_LEASED then
			self:leaseTime()
		end;
	end;

end;


if Vehicle ~= nil and Vehicle.leaseTime == nil then
	Vehicle.leaseTime = function(self)

		if self.propertyState == Vehicle.PROPERTY_STATE_LEASED then
		
			-- To calculate time remaining in lease until next hourly charge.
			local hours = math.floor(self.operatingTime / (1000 * 60 * 60))
			local totalTime = (hours + 1) * (1000 * 60 * 60)
			local remaining = totalTime - self.operatingTime									-- Remaining lease time in ms.
			local minutes = math.floor(remaining / (1000 * 60))									-- Remaining lease time in minutes.
			
			-- To determine the vehicle brand and name.
			local vehicleId = StoreItemsUtil.storeItemsByXMLFilename[self.configFileName:lower()]
			local brand = tostring(Utils.getNoNil(vehicleId.brand,""))
			local name = tostring(Utils.getNoNil(vehicleId.name,""))
			vehicleId = brand .. " " .. name
			
			-- Render remaining lease time for this vehicle in the help menu.
			g_currentMission:addExtraPrintText(string.format(g_i18n:getText("NEXT_CHARGE"), vehicleId, minutes))
		end;
	end;
end;


function VehicleSellingPoint:update(dt)
	
	-- Existing code.
	if self.isEnabled and self.sellIconId ~= nil then
        rotate(self.sellIconId, 0, 0.002 * dt, 0)
    end

	-- Added code -----------------------------------------------------------
	if self.checkSoldVehicle then						-- Did we just attempt to sell/return a vehicle?
		if self.currentVehicle ~= nil then
			if self.currentVehicle.isDeleted then			-- Was the vehicle actually sold/returned?

				-- Sell fillType in sold/returned vehicle.
				local vehicle = self.currentVehicle
				local state = vehicle.propertyState
				local fillUnits = vehicle.fillUnits
				if fillUnits ~= nil then
					for i, unit in pairs(fillUnits) do
						local fillType = Utils.getNoNil(unit.currentFillType, 0)
						local fillLevel = Utils.getNoNil(unit.fillLevel, 0)
						if fillType ~= 0 and fillLevel > 0 then
							local price = fillLevel * g_currentMission.economyManager:getPricePerLiter(fillType) * discount
							fillLevel = math.floor(fillLevel)
							local money = getMoneyTypeId()
							local unitType = g_i18n:getText("UNIT")
							local fillName = FillUtil.fillTypeIntToName[fillType]
							local message1 = "other"
							local message2 = string.format(g_i18n:getText("VEHICLE_SALE_FILLTYPE_SELL_MESSAGE"), tostring(fillLevel), tostring(unitType), tostring(fillName))
							if state == Vehicle.PROPERTY_STATE_LEASED then
								message1 = "vehicleLeasingCost"
								message2 = string.format(g_i18n:getText("LEASE_RETURN_FILLTYPE_SELL_MESSAGE"), tostring(fillLevel), tostring(unitType), tostring(fillName))
							end;

							-- To ensure vehicle is 'cleaned out' in case we ever come this way again.
							self.currentVehicle.fillUnits[i].currentFillType = FillUtil.FILLTYPE_UNKNOWN
							self.currentVehicle.fillUnits[i].fillLevel = 0
							
							-- Activate the refund.
							self:saleLeaseReturn(price, money, message1, message2)
							
							--[[Debug checking.
							print("The vehicle name is: " .. tostring(vehicle.configFileName) .. " with a property state of " .. tostring(vehicle.propertyState))
							print("FillType: " .. tostring(FillUtil.fillTypeIntToName[fillType]) .. " FillLevel: " .. tostring(fillLevel))					
							print("Full price: " .. tostring(fillLevel * g_currentMission.economyManager:getPricePerLiter(fillType)))
							print("Actual price: " .. tostring(math.floor(price)))
							print("Money type: " .. tostring(money))]]
						end;
					end;
				end;

				-- Refill fuel levels in returned/rented vehicle.
				if state == Vehicle.PROPERTY_STATE_LEASED then
					local fillType = FillUtil.FILLTYPE_FUEL
					local fuelCapacity = Utils.getNoNil(vehicle.fuelCapacity, 0)
					local fillLevel = Utils.getNoNil(vehicle.fuelFillLevel, 0)
					if fuelCapacity > 0 and (fillLevel > 0 and fillLevel < fuelCapacity) then	-- if the fuel tank isn't full.
						local delta = fuelCapacity - fillLevel
						if delta > 0 then
							local cost = delta * g_currentMission.economyManager:getPricePerLiter(fillType)
							delta = math.floor(delta)
							local money = getMoneyTypeId()
							local unitType = g_i18n:getText("UNIT")
							local message1 = "vehicleLeasingCost"
							local message2 = string.format(g_i18n:getText("LEASE_RETURN_FUEL_PURCHASE_MESSAGE"), tostring(delta), tostring(unitType))

							-- To ensure vehicle fuel is 'full up' in case we ever come this way again.
							self.currentVehicle.fuelFillLevel = self.currentVehicle.fuelCapacity
							
							-- Activate the payment. 
							self:saleLeaseReturn(-cost, money, message1, message2)
							
							--[[Debug checking.
							print("The vehicle name is: " .. tostring(vehicle.configFileName) .. " with a property state of " .. tostring(vehicle.propertyState))
							print("Fuel price paid: " .. tostring(math.floor(cost)))]]
						end;
					end;
				end;

				-- Reset variables for vehicle sale/return.
				self.checkSoldVehicle = nil
			end;
		end;
	end;
	-------------------------------------------------------------------------------
end;

function VehicleSellingPoint:saleLeaseReturn(amount, money, message1, message2)

	if g_currentMission:getIsServer() then									-- What we want the server to run.
		g_currentMission:addSharedMoney(amount, message1);
		g_currentMission:addMoneyChange(amount, money)						-- No need to show message as this is the server.
		if amount < 0 then
			g_currentMission.missionStats:updateStats("expenses", -amount)
		end;
	else																	-- What we want the client to run.
		g_client:getServerConnection():sendEvent(vehicleSaleLeaseEvent:new(amount, money, message1))
	end;
	
	if g_client ~= nil then
		g_currentMission:showMoneyChange(money, message2)					-- Message to show on client side.
	end;
	
end;

function VehicleSellingPoint:onActivateObject()

	-- Existing code.
    self:determineCurrentVehicle()
    g_gui:showDirectSellDialog({vehicle=self.currentVehicle, owner=self, ownWorkshop=self.ownWorkshop})

	-- Added code.
	self.checkSoldVehicle = true

end;

function VehicleSellingPoint:sellAreaTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    if otherShapeId ~= nil and (onEnter or onLeave) then
        if onEnter then
            self.vehicleInRange[otherShapeId] = true
        elseif onLeave then
            self.vehicleInRange[otherShapeId] = nil
			self.checkSoldVehicle = nil -- Added code:  If we leave the trigger, then stop checking for a sale/return.
        end
        self:determineCurrentVehicle()
    end
end


---------------------------------------------------------------------------------------------------------
--	VEHICLE SALE/LEASE RETURN EVENT FOR MULTIPLAYER
---------------------------------------------------------------------------------------------------------
vehicleSaleLeaseEvent = {};
vehicleSaleLeaseEvent_mt = Class(vehicleSaleLeaseEvent, Event);

InitEventClass(vehicleSaleLeaseEvent, "vehicleSaleLeaseEvent");

function vehicleSaleLeaseEvent:emptyNew()
	local self = Event:new(vehicleSaleLeaseEvent_mt);
	self.className = "vehicleSaleLeaseEvent";
	return self;
end;

function vehicleSaleLeaseEvent:new(amount, money, message1)
	local self = vehicleSaleLeaseEvent:emptyNew()
	self.amount = amount;
	self.money = money;
	self.message1 = message1;
	return self;
end;

function vehicleSaleLeaseEvent:readStream(streamId, connection)
	self.amount = streamReadFloat32(streamId);
	self.money = streamReadInt32(streamId);
	self.message1 = streamReadString(streamId);
	self:run(connection);
end;

function vehicleSaleLeaseEvent:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.amount);
	streamWriteInt32(streamId,self.money);
	streamWriteString(streamId, self.message1);
end;

function vehicleSaleLeaseEvent:run(connection)
	if not connection:getIsServer() then					-- What to do on the server.
		g_currentMission:addSharedMoney(self.amount, self.message1);
		g_currentMission:addMoneyChange(self.amount, self.money)
		if self.amount < 0 then
			g_currentMission.missionStats:updateStats("expenses", -self.amount)
		end;		
	end;
end;